package com.mortals.xhx.module.matter.service.impl;
import cn.hutool.core.util.RandomUtil;
import cn.hutool.http.HttpUtil;
import com.mortals.framework.ap.GlobalSysInfo;
import com.mortals.framework.model.PageInfo;
import com.mortals.framework.util.DateUtils;
import com.mortals.xhx.common.utils.StringUtils;
import com.mortals.xhx.module.matter.model.*;
import com.mortals.xhx.module.matter.service.*;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import com.mortals.framework.service.impl.AbstractCRUDServiceImpl;
import com.mortals.framework.exception.AppException;
import com.mortals.framework.model.Context;
import com.mortals.xhx.module.matter.dao.UserMatterApplyDao;
import org.springframework.util.ObjectUtils;

import java.util.*;
import java.util.stream.Collectors;

import static com.mortals.xhx.common.key.Constant.PARAM_SERVER_PHP_HTTP_URL;

/**
* UserMatterApplyService
* 用户申报信息 service实现
*
* @author zxfei
* @date 2023-04-10
*/
@Service("userMatterApplyService")
public class UserMatterApplyServiceImpl extends AbstractCRUDServiceImpl<UserMatterApplyDao, UserMatterApplyEntity, Long> implements UserMatterApplyService {

    @Autowired
    private UserMatterDetailsService userMatterDetailsService;
    @Autowired
    private MatterApplyService matterApplyService;
    @Autowired
    private MatterDatumService matterDatumService;
    @Autowired
    private MatterDatumFileService matterDatumFileService;

    @Override
    protected void findAfter(UserMatterApplyEntity params, Context context, List<UserMatterApplyEntity> list) throws AppException {

        list.stream().peek(item->{
            List<UserMatterDetailsEntity> matterDetailsFileEntities = userMatterDetailsService.find(new UserMatterDetailsQuery().applyrId(item.getId()));
            item.setUserMatterDetails(matterDetailsFileEntities);
            //item.setApplyPerson(desensitizedName(item.getApplyPerson()));
        }).count();
        super.findAfter(params, context, list);
    }

    @Override
    protected void findAfter(UserMatterApplyEntity params, PageInfo pageInfo, Context context, List<UserMatterApplyEntity> list) throws AppException {

        list.stream().peek(item->{
            List<UserMatterDetailsEntity> matterDetailsFileEntities = userMatterDetailsService.find(new UserMatterDetailsQuery().applyrId(item.getId()));
            item.setUserMatterDetails(matterDetailsFileEntities);
            //item.setApplyPerson(desensitizedName(item.getApplyPerson()));
        }).count();
        super.findAfter(params, context, list);
    }

    @Override
    protected void saveBefore(UserMatterApplyEntity entity, Context context) throws AppException {
        super.saveBefore(entity,context);
        entity.setApplyCode("SQ"+ new Date().getTime());
        entity.setStatus(0);
        if(entity.getMatterId()==null){
            throw new AppException("申请事项不能为空");
        }else {
            MatterApplyEntity matterApplyEntity = matterApplyService.get(entity.getMatterId());
            if(matterApplyEntity!=null){
                entity.setMatterCode(matterApplyEntity.getMatterCode());
                entity.setMatterName(matterApplyEntity.getMatterName());
                entity.setDeptName(matterApplyEntity.getDeptName());
            }
        }
        entity.setApplyDate(new Date());
        entity.setStoreDate(new Date());
    }

    @Override
    protected void saveAfter(UserMatterApplyEntity entity, Context context) throws AppException {
        if (!ObjectUtils.isEmpty(entity.getUserMatterDetails())) {
            List<UserMatterDetailsEntity> details = doSaveAfer(entity, context);
            userMatterDetailsService.save(details);
            try {
                String datum_name = details.stream().map(UserMatterDetailsEntity::getFullName).collect(Collectors.joining(";"));
                Map<String, Object> paramsMap = new HashMap<>();
                paramsMap.put("userid",entity.getUserId());
                paramsMap.put("datum_name",datum_name);
                paramsMap.put("matter_name",entity.getMatterName());
                paramsMap.put("apply_time",DateUtils.getCurrStrDateTime());
                String phpUrl = GlobalSysInfo.getParamValue(PARAM_SERVER_PHP_HTTP_URL, "http://172.15.28.116:8090");
                phpUrl += "/pickup/enter_api/enterForPeople";
                String resp = HttpUtil.createPost(phpUrl).form(paramsMap).execute().body();

            }catch (Exception e){
                log.error("群众助办件,寄存件信息录入接调用异常",e);
            }
        }
        matterApplyService.addReportCount(entity.getMatterId());
        super.saveAfter(entity, context);
    }

    @Override
    protected void removeAfter(Long[] ids, Context context, int result) throws AppException {
        List<Long> applyIdList = new ArrayList<>();
        for(Long id:ids){
            applyIdList.add(id);
        }
        List<UserMatterDetailsEntity> list = userMatterDetailsService.find(new UserMatterDetailsQuery().applyrIdList(applyIdList));
        List<Long> dataIds = new ArrayList<>();
        for(UserMatterDetailsEntity item:list){
            dataIds.add(item.getId());
        }
        userMatterDetailsService.remove(dataIds.toArray(new Long[dataIds.size()]),context);
    }

    private List<UserMatterDetailsEntity> doSaveAfer(UserMatterApplyEntity entity, Context context) throws AppException{
        List<UserMatterDetailsEntity> details = new ArrayList<>();
        if (!ObjectUtils.isEmpty(entity.getUserMatterDetails())) {
            for(UserMatterDetailsEntity item:entity.getUserMatterDetails()){
                MatterDatumEntity datumEntity = matterDatumService.get(item.getDatumId());
                if(datumEntity==null){
                    throw new AppException("事项材料ID不正确");
                }else {
                    List<MatterDatumFileEntity> datumFileList = matterDatumFileService.find(new MatterDatumFileQuery().datumId(item.getDatumId()));
                    for(MatterDatumFileEntity datumFile:datumFileList){
                        UserMatterDetailsEntity matterDetailsEntity = new UserMatterDetailsEntity();
                        matterDetailsEntity.setApplyrId(entity.getId());
                        matterDetailsEntity.setMatterId(entity.getMatterId());
                        matterDetailsEntity.setMatterCode(entity.getMatterCode());
                        matterDetailsEntity.setDatumId(item.getDatumId());
                        matterDetailsEntity.setFullName(datumEntity.getFullName());
                        matterDetailsEntity.setShortName(datumEntity.getShortName());
                        matterDetailsEntity.setFileName(datumFile.getFileName());
                        if(StringUtils.isNotEmpty(item.getPreviewUrl())){
                            matterDetailsEntity.setPreviewUrl(item.getPreviewUrl());
                        }else {
                            matterDetailsEntity.setPreviewUrl(datumFile.getPreviewUrl());
                        }
                        matterDetailsEntity.setImageBase64(item.getImageBase64());
                        matterDetailsEntity.setTemplateUrl(datumFile.getTemplatePath());
                        matterDetailsEntity.setFormContent(item.getFormContent());
                        matterDetailsEntity.setFormTemplate(datumFile.getFormContent());
                        matterDetailsEntity.setCreateTime(new Date());
                        details.add(matterDetailsEntity);
                    }
                }

            }
        }
        return details;
    }

    private static String desensitizedName(String fullName){
        if (StringUtils.isNotEmpty(fullName)) {
            String name = StringUtils.left(fullName, 1);
            return StringUtils.rightPad(name, StringUtils.length(fullName), "*");
        }
        return fullName;
    }

    @Override
    public int getAllCount() {
        return dao.getCount(new UserMatterApplyQuery());
    }

    @Override
    public int getTodayCount() {
        String dateStr = DateUtils.getDateTimeStr(new Date(),"yyyy-MM-dd");
        UserMatterApplyQuery query = new UserMatterApplyQuery();
        query.setCreateTimeStart(dateStr);
        query.setCreateTimeEnd(dateStr);
        return dao.getCount(query);
    }
}